# ncurses 5.9 - patch  - Thomas E. Dickey
#
# ------------------------------------------------------------------------------
#
# Ncurses 5.9 is at
# 	ftp.gnu.org:/pub/gnu
#
# Patches for ncurses 5.9 are in the subdirectory
# 	ftp://invisible-island.net/ncurses/5.9
#
# ------------------------------------------------------------------------------
# ftp://invisible-island.net/ncurses/5.9/ncurses-5.9-20140823.patch.gz
# patch by Thomas E. Dickey <dickey@invisible-island.net>
# created  Sun Aug 24 00:42:29 UTC 2014
# ------------------------------------------------------------------------------
# NEWS                             |    7 ++++++-
# dist.mk                          |    4 ++--
# ncurses/tty/tty_update.c         |   28 ++++++++++++++++++++--------
# package/debian-mingw/changelog   |    4 ++--
# package/debian-mingw64/changelog |    4 ++--
# package/debian/changelog         |    4 ++--
# package/mingw-ncurses.nsi        |    4 ++--
# package/mingw-ncurses.spec       |    2 +-
# package/ncurses.spec             |    2 +-
# 9 files changed, 38 insertions(+), 21 deletions(-)
# ------------------------------------------------------------------------------
Index: NEWS
Prereq:  1.2260 
--- ncurses-5.9-20140816+/NEWS	2014-08-16 23:28:26.000000000 +0000
+++ ncurses-5.9-20140823/NEWS	2014-08-23 19:28:39.000000000 +0000
@@ -25,7 +25,7 @@
 -- sale, use or other dealings in this Software without prior written        --
 -- authorization.                                                            --
 -------------------------------------------------------------------------------
--- $Id: NEWS,v 1.2260 2014/08/16 23:28:26 tom Exp $
+-- $Id: NEWS,v 1.2262 2014/08/23 19:28:39 tom Exp $
 -------------------------------------------------------------------------------
 
 This is a log of changes that ncurses has gone through since Zeyd started
@@ -45,6 +45,11 @@
 Changes through 1.9.9e did not credit all contributions;
 it is not possible to add this information.
 
+20140823
+	+ fix special case where double-width character overwrites a single-
+	  width character in the first column (report by Egmont Koblinger,
+	  cf: 20050813).
+
 20140816
 	+ fix colors in ncurses 'b' test which did not work after changing
 	  it to put the test-strings in subwindows (cf: 20140705).
Index: dist.mk
Prereq:  1.1001 
--- ncurses-5.9-20140816+/dist.mk	2014-08-14 07:59:48.000000000 +0000
+++ ncurses-5.9-20140823/dist.mk	2014-08-23 16:35:54.000000000 +0000
@@ -25,7 +25,7 @@
 # use or other dealings in this Software without prior written               #
 # authorization.                                                             #
 ##############################################################################
-# $Id: dist.mk,v 1.1001 2014/08/14 07:59:48 tom Exp $
+# $Id: dist.mk,v 1.1002 2014/08/23 16:35:54 tom Exp $
 # Makefile for creating ncurses distributions.
 #
 # This only needs to be used directly as a makefile by developers, but
@@ -37,7 +37,7 @@
 # These define the major/minor/patch versions of ncurses.
 NCURSES_MAJOR = 5
 NCURSES_MINOR = 9
-NCURSES_PATCH = 20140816
+NCURSES_PATCH = 20140823
 
 # We don't append the patch to the version, since this only applies to releases
 VERSION = $(NCURSES_MAJOR).$(NCURSES_MINOR)
Index: ncurses/tty/tty_update.c
Prereq:  1.279 
--- ncurses-5.9-20140816+/ncurses/tty/tty_update.c	2014-07-12 23:16:30.000000000 +0000
+++ ncurses-5.9-20140823/ncurses/tty/tty_update.c	2014-08-23 19:25:18.000000000 +0000
@@ -82,7 +82,7 @@
 
 #include <ctype.h>
 
-MODULE_ID("$Id: tty_update.c,v 1.279 2014/07/12 23:16:30 tom Exp $")
+MODULE_ID("$Id: tty_update.c,v 1.280 2014/08/23 19:25:18 tom Exp $")
 
 /*
  * This define controls the line-breakout optimization.  Every once in a
@@ -628,6 +628,7 @@
 	 int first, int last)
 {
     int i, j, same;
+    int rc;
 
     TR(TRACE_CHARPUT, ("PutRange(%p, %p, %p, %d, %d, %d)",
 		       (void *) SP_PARM,
@@ -655,9 +656,11 @@
 	 * Always return 1 for the next GoTo() after a PutRange() if we found
 	 * identical characters at end of interval
 	 */
-	return (same == 0 ? i : 1);
+	rc = (same == 0 ? i : 1);
+    } else {
+	rc = EmitRange(NCURSES_SP_ARGx ntext + first, last - first + 1);
     }
-    return EmitRange(NCURSES_SP_ARGx ntext + first, last - first + 1);
+    return rc;
 }
 
 /* leave unbracketed here so 'indent' works */
@@ -1492,9 +1495,17 @@
 	    if (oLastChar < nLastChar) {
 		int m = max(nLastNonblank, oLastNonblank);
 #if USE_WIDEC_SUPPORT
-		while (isWidecExt(newLine[n + 1]) && n) {
-		    --n;
-		    --oLastChar;
+		if (n) {
+		    while (isWidecExt(newLine[n + 1]) && n) {
+			--n;
+			--oLastChar;	/* increase cost */
+		    }
+		} else if (n >= firstChar &&
+			   isWidecBase(newLine[n])) {
+		    while (isWidecExt(newLine[n + 1])) {
+			++n;
+			++oLastChar;	/* decrease cost */
+		    }
 		}
 #endif
 		GoTo(NCURSES_SP_ARGx lineno, n + 1);
@@ -1514,8 +1525,9 @@
 		if (DelCharCost(SP_PARM, oLastChar - nLastChar)
 		    > SP_PARM->_el_cost + nLastNonblank - (n + 1)) {
 		    if (PutRange(NCURSES_SP_ARGx oldLine, newLine, lineno,
-				 n + 1, nLastNonblank))
-			  GoTo(NCURSES_SP_ARGx lineno, nLastNonblank + 1);
+				 n + 1, nLastNonblank)) {
+			GoTo(NCURSES_SP_ARGx lineno, nLastNonblank + 1);
+		    }
 		    ClrToEOL(NCURSES_SP_ARGx blank, FALSE);
 		} else {
 		    /*
Index: package/debian-mingw/changelog
--- ncurses-5.9-20140816+/package/debian-mingw/changelog	2014-08-14 07:59:48.000000000 +0000
+++ ncurses-5.9-20140823/package/debian-mingw/changelog	2014-08-23 16:35:54.000000000 +0000
@@ -1,8 +1,8 @@
-ncurses6 (5.9-20140816) unstable; urgency=low
+ncurses6 (5.9-20140823) unstable; urgency=low
 
   * latest weekly patch
 
- -- Thomas E. Dickey <dickey@invisible-island.net>  Thu, 14 Aug 2014 03:59:48 -0400
+ -- Thomas E. Dickey <dickey@invisible-island.net>  Sat, 23 Aug 2014 12:35:54 -0400
 
 ncurses6 (5.9-20131005) unstable; urgency=low
 
Index: package/debian-mingw64/changelog
--- ncurses-5.9-20140816+/package/debian-mingw64/changelog	2014-08-14 07:59:48.000000000 +0000
+++ ncurses-5.9-20140823/package/debian-mingw64/changelog	2014-08-23 16:35:54.000000000 +0000
@@ -1,8 +1,8 @@
-ncurses6 (5.9-20140816) unstable; urgency=low
+ncurses6 (5.9-20140823) unstable; urgency=low
 
   * latest weekly patch
 
- -- Thomas E. Dickey <dickey@invisible-island.net>  Thu, 14 Aug 2014 03:59:48 -0400
+ -- Thomas E. Dickey <dickey@invisible-island.net>  Sat, 23 Aug 2014 12:35:54 -0400
 
 ncurses6 (5.9-20131005) unstable; urgency=low
 
Index: package/debian/changelog
--- ncurses-5.9-20140816+/package/debian/changelog	2014-08-14 07:59:48.000000000 +0000
+++ ncurses-5.9-20140823/package/debian/changelog	2014-08-23 16:35:54.000000000 +0000
@@ -1,8 +1,8 @@
-ncurses6 (5.9-20140816) unstable; urgency=low
+ncurses6 (5.9-20140823) unstable; urgency=low
 
   * latest weekly patch
 
- -- Thomas E. Dickey <dickey@invisible-island.net>  Thu, 14 Aug 2014 03:59:48 -0400
+ -- Thomas E. Dickey <dickey@invisible-island.net>  Sat, 23 Aug 2014 12:35:54 -0400
 
 ncurses6 (5.9-20120608) unstable; urgency=low
 
Index: package/mingw-ncurses.nsi
Prereq:  1.56 
--- ncurses-5.9-20140816+/package/mingw-ncurses.nsi	2014-08-14 07:59:48.000000000 +0000
+++ ncurses-5.9-20140823/package/mingw-ncurses.nsi	2014-08-23 16:35:54.000000000 +0000
@@ -1,4 +1,4 @@
-; $Id: mingw-ncurses.nsi,v 1.56 2014/08/14 07:59:48 tom Exp $
+; $Id: mingw-ncurses.nsi,v 1.57 2014/08/23 16:35:54 tom Exp $
 
 ; TODO add examples
 ; TODO bump ABI to 6
@@ -10,7 +10,7 @@
 !define VERSION_MAJOR "5"
 !define VERSION_MINOR "9"
 !define VERSION_YYYY  "2014"
-!define VERSION_MMDD  "0816"
+!define VERSION_MMDD  "0823"
 !define VERSION_PATCH ${VERSION_YYYY}${VERSION_MMDD}
 
 !define MY_ABI   "5"
Index: package/mingw-ncurses.spec
--- ncurses-5.9-20140816+/package/mingw-ncurses.spec	2014-08-14 07:59:48.000000000 +0000
+++ ncurses-5.9-20140823/package/mingw-ncurses.spec	2014-08-23 16:35:54.000000000 +0000
@@ -3,7 +3,7 @@
 Summary: shared libraries for terminal handling
 Name: mingw32-ncurses6
 Version: 5.9
-Release: 20140816
+Release: 20140823
 License: X11
 Group: Development/Libraries
 Source: ncurses-%{version}-%{release}.tgz
Index: package/ncurses.spec
--- ncurses-5.9-20140816+/package/ncurses.spec	2014-08-14 07:59:48.000000000 +0000
+++ ncurses-5.9-20140823/package/ncurses.spec	2014-08-23 16:35:54.000000000 +0000
@@ -1,7 +1,7 @@
 Summary: shared libraries for terminal handling
 Name: ncurses6
 Version: 5.9
-Release: 20140816
+Release: 20140823
 License: X11
 Group: Development/Libraries
 Source: ncurses-%{version}-%{release}.tgz
